@@ -1,28 +1,20 @@
-# #!/usr/bin/env python
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # <sure - assertion toolbox>
 # Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
 #
-# Permission is hereby granted, free of charge, to any person
-# obtaining a copy of this software and associated documentation
-# files (the "Software"), to deal in the Software without
-# restriction, including without limitation the rights to use,
-# copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following
-# conditions:
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
-# The above copyright notice and this permission notice shall be
-# included in all copies or substantial portions of the Software.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-# OTHER DEALINGS IN THE SOFTWARE.
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import re
 import os
 import sys
@@ -36,10 +28,13 @@ from datetime import datetime
 try:
     from collections import Iterable
 except ImportError:
-    Iterable = (list, dict, tuple)
+    Iterable = (list, dict, tuple, set)
 
 from sure.registry import context as _registry
-version = '0.10.4'
+from sure.magic import is_cpython, patchable_builtin
+
+
+version = '1.0.3'
 
 
 def _get_file_name(func):
@@ -117,7 +112,10 @@ class CallBack(object):
         try:
             return self.callback(*args, **self.kwargs)
         except:
-            err = unicode(sys.exc_info()[1])
+            exc_klass, exc_value, tb = sys.exc_info()
+            err = traceback.format_exc(tb).splitlines()[-1]
+            err = err.replace('{0}:'.format(exc_klass.__name__), '').strip()
+
             if err.startswith(self.callback_name) and \
                'takes no arguments (1 given)' in err:
                 raise TypeError(self.context_error % (
@@ -243,12 +241,12 @@ class that(object):
             self._src(*self._callable_args, **self._callable_kw)
 
         except Exception, e:
-            err = str(e)
-            exc = type(e)
-
             if isinstance(exc, basestring):
                 msg = exc
 
+            err = str(e)
+            exc = type(e)
+
             if isinstance(exc, type) and issubclass(exc, Exception):
                 if not isinstance(e, exc):
                     raise AssertionError(
@@ -396,13 +394,17 @@ class that(object):
         return True
 
     def len_lower_than(self, that):
-        that = self._get_that(that)
+        original_that = that
+        if isinstance(that, Iterable):
+            that = len(that)
+        else:
+            that = self._get_that(that)
         length = len(self._src)
 
         if length >= that:
-            error = 'the length of %r should be lower then %d, but is %d' % (
+            error = 'the length of %r should be lower then %r, but is %d' % (
                 self._src,
-                that,
+                original_that,
                 length,
             )
             raise AssertionError(error)
@@ -439,6 +441,19 @@ class that(object):
 
         return True
 
+    def len_is_not(self, that):
+        that = self._get_that(that)
+        length = len(self._src)
+
+        if length == that:
+            error = 'the length of %r should not be %d' % (
+                self._src,
+                that,
+            )
+            raise AssertionError(error)
+
+        return True
+
     def like(self, that):
         return self.has(that)
 
@@ -892,5 +907,296 @@ def work_in_progress(func):
     return wrapper
 
 
-def it(name, *args, **kw):
-    return _registry.get(name, *args, **kw)
+def assertionmethod(func):
+    @wraps(func)
+    def wrapper(self, *args, **kw):
+        self.stack.append(func.__name__)
+        value = func(self, *args, **kw)
+        msg = u"{0}({1}) failed".format(
+            func.__name__,
+            u", ".join(map(repr, args)),
+            u", ".join([u"{0}={1}".format(k, repr(kw[k])) for k in kw]),
+        )
+        assert value, msg
+        return value
+
+    return wrapper
+
+
+def assertionproperty(func):
+    return property(assertionmethod(func))
+
+POSITIVES = [
+    'should',
+    'does',
+    'do',
+    'must',
+    'when',
+]
+
+NEGATIVES = [
+    'shouldnt',
+    'dont',
+    'do_not',
+    'doesnt',
+    'does_not',
+    'doesnot',
+    'should_not',
+    'shouldnot',
+]
+
+
+class AssertionBuilder(object):
+    def __init__(self, name=None, negative=False):
+        self._name = name
+        self.negative = negative
+        self._callable_args = []
+        self._callable_kw = {}
+        self.stack = []
+        if name:
+            self.stack.append(name)
+
+    def __call__(self, obj):
+        self.stack.append('it')
+
+        if isinstance(obj, self.__class__):
+            obj = obj.obj
+
+        self.obj = obj
+        self._that = that(obj)
+
+        return self
+
+    def __getattr__(self, attr):
+        special_case = False
+        special_case = attr in (POSITIVES + NEGATIVES)
+
+        self.negative = attr in NEGATIVES
+
+        if attr in POSITIVES:
+            self.negative = False
+            special_case = True
+
+        if attr in NEGATIVES:
+            self.negative = True
+            special_case = True
+
+        if special_case:
+            self.stack.append(attr)
+
+            return self
+
+        return super(AssertionBuilder, self).__getattribute__(attr)
+
+    @assertionproperty
+    def be(self):
+        return self
+
+    @assertionproperty
+    def when(self):
+        return self
+
+    @assertionproperty
+    def have(self):
+        return self
+
+    @assertionproperty
+    def empty(self):
+        if self.negative:
+            return self._that.len_greater_than(0)
+        else:
+            return self._that.len_is(0)
+
+    @assertionproperty
+    def ok(self):
+        if self.negative:
+            msg = 'expected `{0}` to be falsy'.format(self.obj)
+            assert not bool(self.obj), msg
+        else:
+            msg = 'expected `{0}` to be truthy'.format(self.obj)
+            assert bool(self.obj), msg
+
+        return True
+
+    truthy = ok
+    true = ok
+
+    @assertionproperty
+    def falsy(self):
+        if self.negative:
+            msg = 'expected `{0}` to be truthy'.format(self.obj)
+            assert bool(self.obj), msg
+        else:
+            msg = 'expected `{0}` to be falsy'.format(self.obj)
+            assert not bool(self.obj), msg
+
+        return True
+
+    false = falsy
+
+    @assertionproperty
+    def none(self):
+        if self.negative:
+            return self.obj is None
+        else:
+            return not self.obj is None
+
+    def __raise(self, prefix="", suffix=""):
+        raise AssertionError('{0}{1}{2}'.format(
+            ".".join(self.stack), self.obj, suffix))
+
+    @assertionmethod
+    def within(self, *args):
+        if not args:
+            return self.__raise(suffix=" re")
+
+        first = args[0]
+        if isinstance(first, Iterable):
+            collection_should = that(first)
+        else:
+            collection_should = that(range(*args))
+
+        if self.negative:
+            return collection_should.does_not_contain(self.obj)
+        else:
+            return collection_should.contains(self.obj)
+
+    @assertionmethod
+    def equal(self, what):
+        if self.negative:
+            return self._that.differs(what)
+        else:
+            return self._that.equals(what)
+
+    eql = equal
+    equals = equal
+
+    @assertionmethod
+    def an(self, klass):
+        if isinstance(klass, basestring):
+            if '.' in klass:
+                items = klass.split('.')
+                first = items.pop(0)
+            else:
+                first = '__builtin__'
+                items = [klass]
+
+            klass = reduce(getattr, items, __import__(first))
+        return self._that.is_a(klass)
+
+    a = an
+
+    @assertionmethod
+    def below(self, num):
+        if self.negative:
+            msg = u"{0} should not be below {1}".format(self.obj, num)
+            assert not self.obj < num, msg
+        else:
+            msg = u"{0} should be below {1}".format(self.obj, num)
+            assert self.obj < num, msg
+
+        return True
+
+    @assertionmethod
+    def above(self, num):
+        if self.negative:
+            msg = u"{0} should not be above {1}".format(self.obj, num)
+            assert not self.obj > num, msg
+        else:
+            msg = u"{0} should be above {1}".format(self.obj, num)
+            assert self.obj > num, msg
+        return True
+
+    @assertionmethod
+    def exists(self, what):
+        if self.negative:
+            return not bool(what)
+
+        return bool(what)
+
+    exist = exists
+
+    @assertionmethod
+    def length_of(self, num):
+        if self.negative:
+            return self._that.len_is_not(num)
+
+        return self._that.len_is(num)
+
+    @assertionmethod
+    def called_with(self, *args, **kw):
+        self._callable_args = args
+        self._callable_kw = kw
+        return self
+
+    called = property(called_with)
+
+    @assertionmethod
+    def throw(self, *args, **kw):
+        _that = that(self.obj,
+                     with_args=self._callable_args,
+                     and_kwargs=self._callable_kw)
+
+        if self.negative:
+            msg = (u"{0} called with args {1} and kwargs {2} should "
+                   "not raise {3} but raised {4}")
+
+            exc = args and args[0] or Exception
+            try:
+                _that.raises(*args, **kw)
+                return True
+            except AssertionError, e:
+                err = msg.format(
+                    self.obj,
+                    self._that._callable_args,
+                    self._that._callable_kw,
+                    exc,
+                    e,
+                )
+                raise AssertionError(err)
+
+        return _that.raises(*args, **kw)
+
+    @assertionmethod
+    def return_value(self, value):
+        return_value = self.obj(*self._callable_args, **self._callable_kw)
+        return this(return_value).should.equal(value)
+
+this = AssertionBuilder('this')
+it = AssertionBuilder('it')
+these = AssertionBuilder('these')
+those = AssertionBuilder('those')
+
+
+if is_cpython:
+    positive_builder = AssertionBuilder(negative=False)
+    negative_builder = AssertionBuilder(negative=True)
+
+    def positive_assertion(name, prop=True):
+        def method(self):
+            positive_builder.stack.append(name)
+            return positive_builder(self)
+
+        method.__name__ = name
+        return (property(method) if prop else method(None))
+
+    def negative_assertion(name, prop=True):
+        def method(self):
+            negative_builder.stack.append(name)
+            return negative_builder(self)
+
+        method.__name__ = name
+        return (property(method) if prop else method(None))
+
+    object_handler = patchable_builtin(object)
+
+    # None does not have a tp_dict associated to its PyObject, so this
+    # is the only way we could make it work like we expected.
+    none = patchable_builtin(None.__class__)
+    for name in POSITIVES:
+        object_handler[name] = positive_assertion(name)
+        none[name] = positive_assertion(name, False)
+
+    for name in NEGATIVES:
+        object_handler[name] = negative_assertion(name)
+        none[name] = negative_assertion(name, False)
